// Estes LTV Scout K1287
//
// Robert F. Nee
// (c) 2014

include <Estes.scad>
include <rocketry.scad>

// ---------------------------------------------------------------------------
// Settings
// ---------------------------------------------------------------------------
 
// Curve resolution
$fn = 100;

// ---------------------------------------------------------------------------
// Parts Assumptions
// ---------------------------------------------------------------------------

offset = 0;
clearance = 0.01;
shoulder_length = 0.5;
tube_color = "tan";
slip = 0.003;
shrink = 0.008;

// ---------------------------------------------------------------------------
// Utility functions
// ---------------------------------------------------------------------------

// ---------------------------------------------------------------------------
// Render a body tube

module body_tube (h, diameter, wall)
{
	difference () {
		cylinder (h=h, r=diameter/2);
		translate ([0,0,-0.01])
		cylinder (h=h + 0.02, r=diameter/2 - wall);
	}
}

// ---------------------------------------------------------------------------
// Half round 

module strip (radius, length)
{
	difference () {
		cylinder (length, radius, radius);
		translate ([-radius, -radius/2, -0.01])
		cube ([radius, radius, length + 0.02], false);
	}
}

// ---------------------------------------------------------------------------
// Rounded Cone

module rounded_cone (length, base_radius, tip_radius, wall)
{
	// Compute the angle of that the side of the cone makes with the base.
	// This is the sum of two angles, the angle between the base and a radial
	// from the base corner to the origin of the nose sphere and this radial
	// and the side of the cone.  The total angle can then be used to length
	// of the conical section and the radius at the top of this section.
	t = atan ((length - tip_radius) / base_radius) + 
		asin (tip_radius/sqrt ((length-tip_radius)*(length-tip_radius) + base_radius * base_radius));

	// echo (length, length - tip_radius + tip_radius * cos (t), base_radius	, tip_radius * sin (t));

	// Draw the cone and sphere using the angle t calculated above	
	union () {
		cylinder (h=shoulder_length, r=base_radius - wall);
		translate ([0,0, shoulder_length])
		cylinder (length - tip_radius + tip_radius * cos (t), base_radius,
			 tip_radius * sin (t));
		translate ([0,0, shoulder_length + length - tip_radius])
		sphere (tip_radius);
	}
}

// ---------------------------------------------------------------------------
// Parts for the LTV Scout
// ---------------------------------------------------------------------------

// Parts assumptions
tube1_diam = 1.637;
tube1_wall = 0.021;

tube2_diam = 1.274;
tube2_wall = 0.021;

// tube3_diam = 1.228;
// tube3_wall = 0.023;
// T52H
tube3_diam = 1.214;
tube3_wall = 0.035;

tube4_diam = 1.385;
tube4_wall = 0.023;

duct_diam = 0.250;

// ---------------------------------------------------------------------------

// Tube 1
module bt_60kc ()
{
	color (tube_color)
	body_tube (12.84, tube1_diam, tube1_wall);
}

module sbt_127gc ()
{
	color (tube_color)
	body_tube (7.26, tube2_diam, tube2_wall);
}

module sbt_123be ()
{
	color (tube_color)
	body_tube (2.5, tube3_diam, tube3_wall);
}

module sbt_139bj ()
{
	color (tube_color)
	body_tube (2.0, tube4_diam, tube4_wall);
}


// ---------------------------------------------------------------------------

module pnc_1287_1 ()
{
	// Length: 2.501

	// Positioning for duct
	offset = sqrt (tube2_diam * tube2_diam - duct_diam * duct_diam);
	angle = atan2 ((tube2_diam - tube3_diam)/2, 1.1860);
	id = tube1_diam - 2*tube1_wall;

	h1 = [
		0.890,
		1.860,
		0.400
	];

	d1 = [
		[tube2_diam + shrink,		tube2_diam + shrink],
		[tube2_diam + shrink,		tube3_diam + shrink],
		[tube3_diam - 2*tube3_wall,	tube3_diam - 2*tube3_wall]
	];	

	color ("white")
	difference () {
		union () {
			shape (h1, d1);

			// Ducts.  The top requires a trimming cylinder
			difference () {
				union () {
					translate ([offset/2, 0, 0]) 
					cylinder (h=0.475, d1=duct_diam, d2=0.310);
		
					translate ([offset/2, 0, 0.475]) 
					cylinder (h=0.250, d1=0.310, d2=0);
			
					rotate (90) {
						translate ([offset/2, 0, 0]) 
						cylinder (h=0.400, d=duct_diam);
						
						translate ([offset/2, 0, 0.300]) 
						cylinder (h=0.590, d1=duct_diam, d2=0.310);

						translate ([offset/2, 0, 0.300 + 0.590 - 0.005]) 
						rotate ([0, -angle, 0])
						cylinder (h=1.000, d1 = 0.310, d2 = duct_diam);

						translate ([offset/2, 0, 0.300 + 0.590 - 0.015]) 
						rotate ([0, -angle, 0])
						cylinder (h=2.001 + 0.1, d=duct_diam);
					}

					rotate (270) {
						translate ([offset/2, 0, 0.890 - 0.250]) 
						cylinder (h=0.250, d1 = 0, d2 = 0.310);

						translate ([offset/2, 0, 0.300 + 0.590 - 0.005]) 
						rotate ([0, -angle, 0])
						cylinder (h=1.000, d1 = 0.310, d2 = duct_diam);

						translate ([offset/2, 0, 0.300 + 0.590 - 0.015]) 
						rotate ([0, -angle, 0])
						cylinder (h=2.001 + 0.1, d=duct_diam);
					}
				}		
				jog (0.890 + 1.860)
				cylinder (h = 1, d = 3);
			}
		}

		// Cutout
		jog ()
		cylinder (h = 0.300, d = tube2_diam - 4*tube2_wall);
		jog (0.300 - 0.002)
		cylinder (h = 1.000, d1 = tube2_diam - 4*tube2_wall, d2 = 0);
	}

	jog (-0.600)
	difference () {
		union () {
			cylinder (h = 0.600, d = tube2_diam - 2*tube2_wall);
			jog (0.600)
			cylinder (h = 0.200, d = tube2_diam - 4*tube2_wall);
		}

		jog ()
		attachment_point (tube2_diam * 2/3);
	}
}

// ---------------------------------------------------------------------------

module pnc_1287_2 ()
{
	// Length: 3.198

	neck = 0.975;
	vent = 0.200;

	vent_offset = sqrt (neck * neck - vent * vent);
	angle = atan2 ((tube3_diam - neck)/2, 1.125);

	offset = sqrt (tube3_diam * tube3_diam - duct_diam * duct_diam);

	h1 = [
		1.125,
		2.025,
		0.400
	];

	d1 = [
		[tube3_diam + shrink,		neck + shrink],
		[neck + shrink,			tube4_diam + shrink],
		[tube4_diam - 2*tube4_wall,	tube4_diam - 2*tube4_wall]
	];	

	length = 0.600;
	tip_radius = 0.125;
	base_radius = 0.280;

	t = atan ((length - tip_radius) / base_radius) + 
		asin (tip_radius/sqrt ((length-tip_radius)*(length-tip_radius) + base_radius * base_radius));

	ph = length - tip_radius + tip_radius * cos (t);
	ptr = tip_radius * sin (t);

	color ("white")
	union () {
		difference () {
			union () {
				shape (h1, d1);

				intersection () {
					cylinder (h = h1[0], d1 = d1[0][0] + 0.1, d2 = d1[0][1] + 0.1);

					// Cone extrusion
					rotate (-45)
					translate ([0, 1.5, 0])
					rotate ([90, 0, 0])
					linear_extrude (height = 3)
					union () {
						// Draw the cone and sphere using the angle t calculated above	
						polygon ([
							[-base_radius,		0],
							[-ptr,			ph],
							[ptr,			ph],
							[base_radius,		0.0]
						]);
						translate ([0, length - tip_radius])
						circle (tip_radius);
					}
				}

				for (a = [90, 270]) {
					rotate (a + 45)
					translate ([vent_offset/2, 0, h1[0]]) 
					rotate ([0, -12, 0])
					cylinder (h = 1, d = vent);

					rotate (a)
					translate ([offset/2, 0, -0.015]) 
					rotate ([0, -angle, 0])
					cylinder (h = 0.200, d1 = duct_diam, d2 = 0);
					
				}
			}
	
			jog ()
			cylinder (h = 0.500, d = 1.00);
		
			// Trim bottom of ductwork
			jog (-0.100)
			cylinder (h = 0.100, d = 2.00);
		}

	}

	// Shoulder
	jog (-0.400)
	union () {
		cylinder (h = 0.400, d = tube3_diam - 2*tube3_wall);

		jog (0.400 - 0.001)
		cylinder (h = 0.300, d = 1.000);
	}
}

// Nose cone

module pnc_1287_3 ()
{
	height		= 1.195;	
	round_radius	= 0.316;
	etch_height	= 0.960;

	color ("white")
	etch (0.98) {
		rounded_cone (height, (tube4_diam + shrink)/2, round_radius, tube4_wall);
		
		jog (shoulder_length + etch_height)
		cylinder (h = 0.020, d = tube4_diam);
	}
}

// ---------------------------------------------------------------------------
// Nose cones and transitions

module pta_1287_2 ()
{
	// Length: 2.396

	offset = sqrt (tube1_diam * tube1_diam - duct_diam * duct_diam);
	angle = atan2 ((tube1_diam - tube2_diam)/2, 2.001);
	id = tube1_diam - 2*tube1_wall;

	// Body
	h1 = [
		0.395,
		2.001,
		shoulder_length
	];

	d1 = [
		[tube1_diam,				tube1_diam],
		[tube1_diam,				tube2_diam],
		[tube2_diam - 2*tube2_wall, 	tube2_diam - 2*tube2_wall]
	];	

	difference () {
		// body
		union () {
			chamfered_shell (h1, d1, tube1_diam, 0.060);
		
			// Ducts.  The top requires a trimming cylinder
			difference () {
				for (a = [0, 90]) {
					rotate (a) {
						translate ([offset/2, 0, 0]) 
						cylinder (h=0.395, d1=duct_diam, d2=0.310);
			
						translate ([offset/2, 0, 0.395 - 0.015]) 
						rotate ([0, -angle, 0])
						cylinder (h=0.395, d1=0.310, d2=duct_diam);
			
						translate ([offset/2, 0, 0.395 - 0.015]) 
						rotate ([0, -angle, 0])
						cylinder (h=2.001 + 0.1, d=duct_diam);
					}
				}
		
				jog (0.395 + 2.001)
				cylinder (h = 1, d = 3);
			}
		
			// Dots
			for (a = [0, 90, 180, 270]) {
				rotate (45 + a) {
					translate ([0, 0, 0.395]) 
		
					rotate ([0, -angle, 0])
					translate ([offset/2, 0, 1.125 - 0.395]) 
		
					rotate ([0, 90, 0])
					cylinder (h=0.020, d1=0.200, d2=0.180);
				}
			}
		}

		// Cutout to accept lower shoulder
		jog ()
		cylinder (h = 0.350, d = id - 0.040);

		jog (0.350 - 0.002)
		cylinder (h = 0.050,
			d1 = id - 0.040,
			d2 = id - 0.040 - 2 * 0.050
		);
	}

	// Lower shoulder
	h2 = [
		shoulder_length,
		0.250
	];

	d2 = [
		[tube1_diam - 2*tube1_wall,		 tube1_diam - 2*tube1_wall],
		[tube1_diam - 2*tube1_wall - 0.040, tube1_diam - 2*tube1_wall - 0.040]
	];	

	jog (-1.800)
	chamfered_shell (h2, d2, tube1_diam, 0.060);
}


// ---------------------------------------------------------------------------

module internal_chamfer (h, d1, d2)
{
	difference () {
		cylinder (h = h, d = max (d1, d2));
		jog ()
			cylinder (h = h + 0.002, d1 = d1, d2 = d2);
	}
}

// Lower fin base

module shape (h, d)
{
	for (i = [0 : len (d) - 1]) {
		stack (h, i)
			cylinder (h=height (h, i) + 0.001, d1=d[i][0], d2=d[i][1]);
	}
}


module vertical_shell (s)
{
	difference () {
		children ();
		jog (-0.001)
		scale ([s, s, 1.004])
			children ();
	}
}

module chamfered_shell (h, d, od, thick)
{
	vertical_shell ((od - 2*thick)/od)
		shape (h, d);

	// Attempt to detect and generate any necessary internal chamfers
	for (i = [1 : len (d) - 1]) {
		if (d[i][0] - d[i - 1][1] > thick/2)
//			color ("red")
			stack (h, i)
				internal_chamfer (h = thick, d1 = d[i - 1][1] - 2*thick, d2 = d[i][0] - thick);
		if (d[i - 1][0] - d[i][1] > thick/2)
//			color ("blue")
			stack (h, i)
			jog (-thick)
				internal_chamfer (h = thick, d1 = d[i - 1][1] - 2*thick, d2 = d[i][0] - 2*thick);
	}

}

// ---------------------------------------------------------------------------

module pta_1287_1 ()
{
	thick = 0.050;

	h1 = [
		0.100,
		0.125,
		1.200,
	];

	d1 = [
		[BT60[OD] - 0.160 - 0.250,	BT60[OD] - 0.160],
		[BT60[OD] - 0.160,			BT60[OD] - 0.160],
		[BT60[OD],				BT60[OD]],
	];	

	offset = sqrt (tube1_diam * tube1_diam - duct_diam * duct_diam);

	// Aft half
//	rotate ([180,0,0])
	difference () {
		union () {
			chamfered_shell (h1, d1, BT60[OD], thick);
	
			// Duct caps
			stack (h1, 3)
			for (a = [0, 90]) {
				rotate (a)
				translate ([offset/2, 0, -0.375]) 
				cylinder (h=0.375, d1=0.310, d2=0.250);
			}
		}

		// cutout to trim back of ductwork
		stack (h1, 3)
		jog (-0.400 + slip)
		cylinder (h = 0.400, d = BT60[OD] - 2*thick);
			
		// cutout for forward half
		stack (h1, 3)
		jog (-0.300 + slip)
		cylinder (h = 0.300, d = BT60[ID] - thick/2 - slip);
	}

	// forward half
	h2 = [
		0.200,
		0.350,
		0.150,
	];

	d2 = [
		[BT60[ID] - thick/2,		BT60[ID] - thick/2],
		[BT60[ID],				BT60[ID]],
		[BT60[ID],				BT60[ID] - 0.300],
	];	

	jog (1.5)
	chamfered_shell (h2, d2, BT60[OD], thick);
}

// ---------------------------------------------------------------------------

module prp_1287 ()
{
	h 			= 0.350;
	od			= BT60[OD];
	id			= 0.750;
	outer_wall	= 0.080;
	wall_thick	= 0.060;
	motor_clip	= 0.160;
	notch		= 0.120;
	screw_hole	= 0.075;
	screw_angle	= 21;

	// Derived values
	hole_height	= h + 0.050;
	slot_offset	= od/2 - outer_wall/2;
	pierce_height	= hole_height - slot_offset * tan (screw_angle);	
	chamfer_inset	= h/2 * tan (screw_angle);

	difference () {
		union () {
			// Outer body
			tube (h = h, d = od, t = outer_wall);
		
			// Inner wall
			cylinder (h = wall_thick * 3, d = id + 2 * wall_thick);
		
			// Base
			cylinder (h = wall_thick, d = BT60[OD]);
			
			// Chamfer
			internal_chamfer (h = h/2, d1 = od - 2*(outer_wall + chamfer_inset), d2 = od - 2 * outer_wall);
		}
	
		// center hole
		jog ()
		cylinder (h = h, d = id);

		// Motor clip
		jog ()
		translate ([id/2 - 0.010, -motor_clip/2, 0])
		cube ([motor_clip, motor_clip, h]);

		// Alignment notch
		translate ([-(od - outer_wall)/2, -notch/2, h/2])
		cube ([notch, notch, h]);
	
		// Fin slots
		jog ()
		for (a = [0:90:270]) {
			rotate (45 + a)
			translate ([slot_offset, -notch/2, 0])
			cube ([notch, notch, h + 0.002]);
		}		

		// Screw holes
		for (a = [0:90:270]) {
			rotate (45 + a)
			translate ([0, 0, hole_height])
			rotate ([0, 90 + screw_angle, 0])
			cylinder (h = 1, d = screw_hole);
		}		

	}
}

// ---------------------------------------------------------------------------
// Triangular prism with clipped knife edge
// length		- length of opposite and adjacent sides
// z			- height along edge
// h			- height at clipped edge

module prism (length, z, h)
{
	// Compute additional length of unclipped side over requested length
	t = h * length / (z - h);

	points = [
		[0, 0, 0],
		[0, 0, z],
		[0, length + t, 0],
		[length + t, 0, 0]
	];
	faces = [
		[0, 2, 1],
		[0, 1, 3],
		[1, 2, 3],
		[0, 3, 2]
	];

	difference () {
		polyhedron (points, faces);
	
		// clipping cube
		translate ([length, 0, -0.001]) {
			rotate (45)
			translate ([0, -2 * length, 0])
			cube ([2 * length, 4 * length, z]);
		}
	}
}

// ---------------------------------------------------------------------------
 
module vane ()
{
	angle = 17.0;

	polygon ([
		[0, 0],
		[0.280 * tan (angle), 0.280],
		[0.310 + 0.160 * tan (angle), 0.160],
		[0.310, 0]
	]);
}

module fin_half ()
{
	height = 0.100;
	length = 1.500;
	angle = 17;
	angle2 = 18;
	notch = 0.110;
	cutout = 0.050;
	tab_length = 0.350;
	tab_depth = 0.040;
	etch_x = 1.500 - 0.350;

	// Derived values from prp_1287 to locate screw hole
	screw_hole	= 0.065;
	screw_angle	= 21;
	od			= tube1_diam;
	outer_wall	= 0.080;
	hole_height	= tab_length + 0.050;
	slot_offset	= od/2 - outer_wall/2;
	pierce_height	= hole_height - slot_offset * tan (screw_angle);	

	difference () {
		union () {
			difference () {
				prism (length, height, 0.010);
		
				// Etch
				translate ([etch_x, -slip, 0.024])
				rotate ([-3.4, 0, 0])
				cube ([0.020, 1, 0.020]);
		
				// base cutout
				jog () {
					translate ([-0.001, -0.001, -0.001])
					cube ([cutout, 0.320 - cutout, 1]);
			
					// angled portion
					translate ([cutout, 0.320 - cutout, 0])
					rotate (45)
					translate ([-cutout, 0, 0])
					cube ([cutout, 0.320 - cutout, 1]);
				}
		
			}
		
			// mount tab
			translate ([-tab_depth, 0.320, 0]) 
			cube ([tab_depth, tab_length, notch/2 - 3 * slip]);
		}
	
		// screw hole
		translate ([-tab_depth, 0.320 + pierce_height,0])
		rotate ([0, 0, -screw_angle])
		translate ([-0.50, 0, 0])
		rotate ([0, 90, 0])
		cylinder (h = 1.0, d = screw_hole);
	}

	// vane tab
	translate ([0.090 + cutout, 0, 0]) {
		translate ([-0.20, 0.080, 0]) 
		cube ([0.200, 0.200, notch/2]);
	
		// vane
		translate ([-0.113, 0, 0]) {
			// Tip
			mirror ([1, 0, 0])
			rotate (-angle2)
			difference () {
				rotate (angle2)
				linear_extrude (notch/2)
				vane ();
		
				translate ([-0.5, 0, notch/2 + 3 * slip])
				rotate ([-12, 0, 0])
				cube (1, 1, 1);
			}
		
			// square edge
			difference () {
				linear_extrude (0.100)
				mirror ([1, 0, 0])
				vane ();
		
				rotate (angle)
				translate ([-1.06, 0, 0])
				cube (1, 1, 1);
			}
		}
	}
}

// ---------------------------------------------------------------------------


module ltv_scout ()
{
	// Jog heights for assembled view
	h_assembled = [
		0.350 - 0.250,
		1.556 - 0.150,
		12.840,
		2.396,
		7.260,
		2.750,
		2.500,
		3.150,
		2.000- 0.5
	];

	h = h_assembled;

	color ("white")
	prp_1287 ();

	stack (h, 1)		
	color ("white")
	pta_1287_1 ();
	
	stack (h, 2)
	bt_60kc (); 

	stack (h, 3)
	color ("white")
	pta_1287_2 ();

	stack (h, 4)
	sbt_127gc ();

	stack (h, 5)
	pnc_1287_1 ();
	stack (h, 6)
	sbt_123be ();
	stack (h, 7)
	pnc_1287_2 ();
	stack (h, 8)
	sbt_139bj ();
	stack (h, 9)
	pnc_1287_3 ();

	// Wiring conduits
	color ("wheat")
	stack (h, 2)
	for (a = [0, 90]) {
		rotate (a)
		translate ([tube1_diam/2, 0, 0])
		strip (0.25/2, h[2]);
	}

	// Wiring conduits
	color ("wheat")
	stack (h, 4)
	for (a = [0, 90]) {
		rotate (a)
		translate ([tube2_diam/2, 0, 0])
		strip (0.25/2, h[4]);
	}

	// Wiring conduits
	color ("wheat")
	stack (h, 6)
	for (a = [90, 270]) {
		rotate (a)
		translate ([tube3_diam/2, 0, 0])
		strip (0.25/2, h[6]);
	}

	// Fins
	color ("white")
	for (i = [0 : 3]) {
		rotate (45 + i* 90)
		translate ([tube1_diam/2, 0, -0.320])
		rotate ([90, 0, 0]) {
			fin_half ();
//			rotate ([180,0,0])
			mirror ([0, 0, 1])
			fin_half ();
		}
	}
}

ltv_scout ();

